@@ -37,6 +37,7 @@ import java.util.Vector;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import javax.xml.bind.DatatypeConverter;
 import javax.xml.transform.OutputKeys;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerFactory;
@@ -58,20 +59,19 @@ import com.verisign.epp.util.EPPCatFactory;
  * Provides a set of utility static methods for use by the EPP Codec classes.
  */
 public class EPPUtil {
-	/** Format used for the XML Schema timeInstant data type with milliseconds. */
-	public static final String TIME_INSTANT_FORMAT = "yyyy-MM-dd'T'HH':'mm':'ss'.'SSSS'Z'";
+	/** Default format used for the XML Schema <code>dateTime</code> data type */
+	public static final String DEFAULT_TIME_INSTANT_FORMAT = "yyyy-MM-dd'T'HH':'mm':'ss'.'SSS'Z'";
 
 	/**
-	 * Format used for the XML Schema timeInstant data type without
-	 * milliseconds.
+	 * Format used for the XML Schema <code>dateTime</code> data type in the
+	 * {@link #encodeTimeInstant(Date) and #encodeTimeInstant(Document, Element,
+	 * Date, String, String)} methods supported by the {@link SimpleDateFormat}.
 	 */
-	public static final String TIME_INSTANT_NO_MILLI_FORMAT = "yyyy-MM-dd'T'HH':'mm':'ss'Z'";
+	private static String timeInstantFormat = DEFAULT_TIME_INSTANT_FORMAT;
 
 	/** Format used for the XML Schema date data type. */
 	public static final String DATE_FORMAT = "yyyy-MM-dd";
 
-	private static Pattern timeInstantPattern;
-
 	/** Log4j category for logging */
 	private static Logger cat = Logger.getLogger(EPPUtil.class.getName(),
 			EPPCatFactory.getInstance().getFactory());
@@ -83,10 +83,33 @@ public class EPPUtil {
 	private static DecimalFormatSymbols decimalFormatSymbols = new DecimalFormatSymbols(
 			Locale.US);
 
-	static {
-		// Pre-compile the timeinstant regular expression
-		timeInstantPattern = Pattern
-				.compile("\\d{4}-\\d{1,2}-\\d{1,2}T\\d{1,2}:\\d{1,2}:\\d{1,2}(\\.\\d{1,4})?Z");
+	/**
+	 * Gets the XML Schema <code>timeDate</code> format used in
+	 * {@link #encodeTimeInstant(Date) and #encodeTimeInstant(Document, Element,
+	 * Date, String, String)}. The defauLt format is defined by the constant
+	 * {@link #DEFAULT_TIME_INSTANT_FORMAT} and it can be overridden using the
+	 * {@link #setTimeInstantFormat(String)} method.
+	 * 
+	 * @return XML Schema <code>timeDate</code> format used in
+	 *         {@link #encodeTimeInstant(Date) and #encodeTimeInstant(Document,
+	 *         Element, Date, String, String)}.
+	 */
+	public static String getTimeInstantFormat() {
+		return timeInstantFormat;
+	}
+
+	/**
+	 * Sets the XML Schema <code>timeDate</code> format used in
+	 * {@link #encodeTimeInstant(Date) and #encodeTimeInstant(Document, Element,
+	 * Date, String, String)}. The format must follow the format supported by
+	 * {@link SimpleDateFormat}.
+	 * 
+	 * @param aTimeInstantFormat
+	 *            XML Schema <code>timeDate</code> format supported by
+	 *            {@link SimpleDateFormat}.
+	 */
+	public static void setTimeInstantFormat(String aTimeInstantFormat) {
+		timeInstantFormat = aTimeInstantFormat;
 	}
 
 	/**
@@ -136,9 +159,28 @@ public class EPPUtil {
 
 		Element theElm = EPPUtil.getElementByTagNameNS(aElement, aNS, aTagName);
 
+		return EPPUtil.decodeBigDecimal(theElm);
+	}
+
+	/**
+	 * Decode <code>BigDecimal</code>, from an XML Element. The
+	 * <code>aElement</code> element will be decoded and returned. If element is
+	 * <code>null</code>, <code>null</code> is returned.
+	 * 
+	 * @param aElement
+	 *            XML Element to decode.
+	 * @return <code>BigDecimal</code> value if <code>aElement</code> is not
+	 *         <code>null</code>; <code>null</code> otherwise.
+	 * @exception EPPDecodeException
+	 *                Error decoding <code>aElement</code>.
+	 */
+	public static BigDecimal decodeBigDecimal(Element aElement)
+			throws EPPDecodeException {
+
 		BigDecimal retBigDecimal = null;
-		if (theElm != null) {
-			Node textNode = theElm.getFirstChild();
+
+		if (aElement != null) {
+			Node textNode = aElement.getFirstChild();
 
 			// Element does have a text node?
 			if (textNode != null) {
@@ -150,14 +192,12 @@ public class EPPUtil {
 					retBigDecimal = new BigDecimal(tempDouble.doubleValue());
 					retBigDecimal = retBigDecimal.setScale(2,
 							BigDecimal.ROUND_HALF_UP);
-				}
-				catch (NumberFormatException e) {
+				} catch (NumberFormatException e) {
 					throw new EPPDecodeException(
 							"Can't convert value to Double: " + doubleValStr
 									+ e);
 				}
-			}
-			else {
+			} else {
 				throw new EPPDecodeException(
 						"Can't decode numeric value from non-existant text node");
 			}
@@ -165,8 +205,6 @@ public class EPPUtil {
 		return retBigDecimal;
 	}
 
-	// End EPPUtil.decodeDate(String)
-
 	/**
 	 * Decode <code>Boolean</code>, by XML namespace and tag name, from an XML
 	 * Element. The children elements of <code>aElement</code> will be searched
@@ -204,12 +242,10 @@ public class EPPUtil {
 
 				if (theVal.equalsIgnoreCase("true") || theVal.equals("1")) {
 					retVal = new Boolean(true);
-				}
-				else {
+				} else {
 					retVal = new Boolean(false);
 				}
-			}
-			else {
+			} else {
 				retVal = null;
 			}
 		}
@@ -245,11 +281,9 @@ public class EPPUtil {
 
 		if (theAttrVal.equals("1") || theAttrVal.equals("true")) {
 			theRet = true;
-		}
-		else if (theAttrVal.equals("0") || theAttrVal.equals("false")) {
+		} else if (theAttrVal.equals("0") || theAttrVal.equals("false")) {
 			theRet = false;
-		}
-		else {
+		} else {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeBooleanAttr: Invalid boolean attr " + aAttr
 							+ " value of " + theAttrVal);
@@ -298,12 +332,10 @@ public class EPPUtil {
 
 				retVal.decode(theElm);
 			}
-		}
-		catch (IllegalAccessException e) {
+		} catch (IllegalAccessException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeComp(), IllegalAccessException: " + e);
-		}
-		catch (InstantiationException e) {
+		} catch (InstantiationException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeComp(), InstantiationException: " + e);
 		}
@@ -357,12 +389,10 @@ public class EPPUtil {
 			}
 
 			// end for each element
-		}
-		catch (IllegalAccessException e) {
+		} catch (IllegalAccessException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeCompList(), IllegalAccessException: " + e);
-		}
-		catch (InstantiationException e) {
+		} catch (InstantiationException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeCompList(), InstantiationException: " + e);
 		}
@@ -416,12 +446,10 @@ public class EPPUtil {
 			}
 
 			// end for each element
-		}
-		catch (IllegalAccessException e) {
+		} catch (IllegalAccessException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeCompVector(), IllegalAccessException: " + e);
-		}
-		catch (InstantiationException e) {
+		} catch (InstantiationException e) {
 			throw new EPPDecodeException(
 					"EPPUtil.decodeCompVector(), InstantiationException: " + e);
 		}
@@ -473,6 +501,7 @@ public class EPPUtil {
 	 * @return Java Date object.
 	 */
 	public static Date decodeDate(String aDateValue) {
+
 		SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT);
 
 		// Set to UTC with no time element
@@ -524,14 +553,12 @@ public class EPPUtil {
 				String intValStr = textNode.getNodeValue();
 				try {
 					retInteger = Integer.valueOf(intValStr);
-				}
-				catch (NumberFormatException e) {
+				} catch (NumberFormatException e) {
 
 					throw new EPPDecodeException(
 							"Can't convert value to Integer: " + intValStr + e);
 				}
-			}
-			else {
+			} else {
 				throw new EPPDecodeException(
 						"Can't decode numeric value from non-existant text node");
 			}
@@ -578,8 +605,7 @@ public class EPPUtil {
 			// Element has text?
 			if (currVal != null) {
 				retVal.add(currVal.getNodeValue());
-			}
-			else { // No text in element.
+			} else { // No text in element.
 				retVal.add("");
 			}
 		}
@@ -620,8 +646,7 @@ public class EPPUtil {
 			// Element does have a text node?
 			if (textNode != null) {
 				retVal = textNode.getNodeValue();
-			}
-			else {
+			} else {
 				retVal = "";
 			}
 		}
@@ -630,6 +655,30 @@ public class EPPUtil {
 		return retVal;
 	}
 
+	/**
+	 * Decode <code>String</code> value from an XML Element.
+	 * 
+	 * @param aElement
+	 *            XML Element to get the <code>String</code> value.
+	 * @return <code>String</code> value if set; &quot;&quot; other.
+	 * @exception EPPDecodeException
+	 *                Error decoding <code>aElement</code>.
+	 */
+	public static String decodeStringValue(Element aElement) {
+		String retVal = null;
+
+		Node textNode = aElement.getFirstChild();
+
+		// Element does have a text node?
+		if (textNode != null) {
+			retVal = textNode.getNodeValue();
+		} else {
+			retVal = "";
+		}
+
+		return retVal;
+	}
+
 	/**
 	 * Decode <code>Date</code>, as date and time, by XML namespace and tag
 	 * name, from an XML Element. The children elements of <code>aElement</code>
@@ -669,10 +718,7 @@ public class EPPUtil {
 	}
 
 	/**
-	 * Decode an XML Schema timeInstant data type to a Java Date object. The XML
-	 * Schema timeInstant data type string has the following format:
-	 * CCYY-MM-DDThh:mm:ss.ssss can be followed by a Z to indicate Coordinated
-	 * Universal Time
+	 * Decode an XML Schema timeInstant data type to a Java Date object.
 	 * 
 	 * @param aTimeInstant
 	 *            XML Schema timeInstant data type string.
@@ -680,30 +726,16 @@ public class EPPUtil {
 	 *         <code>null</code> otherwise
 	 */
 	public static Date decodeTimeInstant(String aTimeInstant) {
+		Date theDate = null;
 
-		Date theRetDate = null;
-
-		Matcher theMatcher = timeInstantPattern.matcher(aTimeInstant);
-
-		// Is the aTimeInstance a valid format?
-		if (theMatcher.matches()) {
-			TimeZone theUtcTimeZone = TimeZone.getTimeZone("UTC");
-			SimpleDateFormat theFormatter;
-
-			// Milliseconds specified?
-			if (theMatcher.group(1) != null) {
-				theFormatter = new SimpleDateFormat(TIME_INSTANT_FORMAT);
-			}
-			else {
-				theFormatter = new SimpleDateFormat(
-						TIME_INSTANT_NO_MILLI_FORMAT);
-			}
-
-			theFormatter.setTimeZone(theUtcTimeZone);
-			theRetDate = theFormatter.parse(aTimeInstant, new ParsePosition(0));
+		try {
+			theDate = DatatypeConverter.parseDateTime(aTimeInstant).getTime();
+		} catch (IllegalArgumentException ex) {
+			cat.error("Exception decoding dateTime \"" + aTimeInstant + "\": " + ex);
+			theDate = null;
 		}
 
-		return theRetDate;
+		return theDate;
 	}
 
 	/**
@@ -745,8 +777,7 @@ public class EPPUtil {
 			// Element has text?
 			if (currVal != null) {
 				retVal.addElement(currVal.getNodeValue());
-			}
-			else { // No text in element.
+			} else { // No text in element.
 				retVal.addElement("");
 			}
 		}
@@ -774,6 +805,10 @@ public class EPPUtil {
 	 *            Tag name of the element including an optional namespace
 	 *            prefix. For example, the tag name for the domain name is
 	 *            "domain:name".
+	 * @param aFormat
+	 *            Format to use following the {@link DecimalFormat} format. If
+	 *            <code>null</code> is passed, the default format of
+	 *            &quot;#0.00&quot; is used.
 	 * @exception EPPEncodeException
 	 *                Error encoding the <code>Double</code>.
 	 */
@@ -792,6 +827,41 @@ public class EPPUtil {
 		}
 	}
 
+	/**
+	 * Encode a <code>Double</code> in XML with a given XML namespace and tag
+	 * name. Takes an optional argument for the format of the Double, if not
+	 * provided then it uses #0.00 which formats the double to 2 decimal places.
+	 * 
+	 * @param aElement
+	 *            XML Element to add the text node encoded with the
+	 *            <code>BigDecimal</code> value.
+	 * @param aBigDecimal
+	 *            <code>Double</code> to encode into the text node of the
+	 *            element.
+	 * @param aFormat
+	 *            Format to use following the {@link DecimalFormat} format. If
+	 *            <code>null</code> is passed, the default format of
+	 *            &quot;#0.00&quot; is used.
+	 * @exception EPPEncodeException
+	 *                Error encoding the <code>Double</code>.
+	 */
+	public static void encodeBigDecimal(Document aDocument, Element aElement,
+			BigDecimal aBigDecimal, String aFormat) throws EPPEncodeException {
+
+		if (aBigDecimal != null) {
+
+			String format = aFormat != null ? aFormat : "#0.00";
+			DecimalFormat decFormat = new DecimalFormat(format,
+					decimalFormatSymbols);
+
+			String numberStr = decFormat.format(aBigDecimal);
+
+			// Add text node
+			Text currVal = aDocument.createTextNode(numberStr);
+			aElement.appendChild(currVal);
+		}
+	}
+
 	/**
 	 * Encode a <code>Boolean</code> in XML with a given XML namespace and tag
 	 * name. If aBoolean is <code>null</code> an element is not added.
@@ -824,8 +894,7 @@ public class EPPUtil {
 
 			if (aBoolean.booleanValue()) {
 				xmlValue = "1";
-			}
-			else {
+			} else {
 				xmlValue = "0";
 			}
 
@@ -852,8 +921,7 @@ public class EPPUtil {
 			boolean aVal) {
 		if (aVal) {
 			aElement.setAttribute(aAttr, "1");
-		}
-		else {
+		} else {
 			aElement.setAttribute(aAttr, "0");
 		}
 	}
@@ -931,7 +999,8 @@ public class EPPUtil {
 	 *            XML Element to add children nodes to. For example, the root
 	 *            node could be &ltdomain:update&gt
 	 * @param aVector
-	 *            <code>Vector</code> of <code>EPPCodecComponent's</code> to add.
+	 *            <code>Vector</code> of <code>EPPCodecComponent's</code> to
+	 *            add.
 	 * @exception EPPEncodeException
 	 *                Error encoding the <code>Vector</code> of
 	 *                <code>EPPCodecComponent</code>'s.
@@ -1141,7 +1210,7 @@ public class EPPUtil {
 	 */
 	public static String encodeTimeInstant(Date aDate) {
 		TimeZone utcTimeZone = TimeZone.getTimeZone("UTC");
-		SimpleDateFormat formatter = new SimpleDateFormat(TIME_INSTANT_FORMAT);
+		SimpleDateFormat formatter = new SimpleDateFormat(timeInstantFormat);
 		formatter.setTimeZone(utcTimeZone);
 
 		return formatter.format(aDate);
@@ -1265,8 +1334,7 @@ public class EPPUtil {
 			}
 
 			return true;
-		}
-		else {
+		} else {
 			return false;
 		}
 	} // End EPPUtil.equalLists(List, List)
@@ -1307,8 +1375,7 @@ public class EPPUtil {
 			}
 
 			return true;
-		}
-		else {
+		} else {
 			return false;
 		}
 	} // End EPPUtil.equalVectors(Vector, Vector)
@@ -1551,8 +1618,7 @@ public class EPPUtil {
 			Element elm = aComponent.encode(document);
 
 			ret = toString(elm);
-		}
-		catch (Exception ex) {
+		} catch (Exception ex) {
 			ex.printStackTrace();
 		}
 
@@ -1583,8 +1649,36 @@ public class EPPUtil {
 					new StreamResult(theBuffer));
 			theBuffer.close();
 			ret = new String(theBuffer.toByteArray());
+		} catch (Exception ex) {
+			ex.printStackTrace();
 		}
-		catch (Exception ex) {
+
+		return ret;
+	}
+
+	/**
+	 * Converts an <code>aElement</code> to a <code>String</code> for printing
+	 * without pretty printing.
+	 * 
+	 * @param aElement
+	 *            <code>Element</code> to print
+	 * @return <code>String</code> representation of <code>Element</code> if
+	 *         successful; "ERROR" <code>String</code> otherwise.
+	 */
+	public static String toStringNoIndent(Element aElement) {
+		String ret = "ERROR";
+		ByteArrayOutputStream theBuffer = new ByteArrayOutputStream();
+
+		// Serialize DOM Document to stream
+		try {
+			TransformerFactory transFac = TransformerFactory.newInstance();
+			Transformer trans = transFac.newTransformer();
+			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
+			trans.transform(new DOMSource(aElement),
+					new StreamResult(theBuffer));
+			theBuffer.close();
+			ret = new String(theBuffer.toByteArray());
+		} catch (Exception ex) {
 			ex.printStackTrace();
 		}
 
@@ -1663,15 +1757,13 @@ public class EPPUtil {
 				String intValStr = textNode.getNodeValue();
 				try {
 					retInteger = Integer.valueOf(intValStr);
-				}
-				catch (NumberFormatException e) {
+				} catch (NumberFormatException e) {
 
 					throw new EPPDecodeException(
 							"EPPUtil.decodeIntegerList Can't convert value to Integer: "
 									+ intValStr + e);
 				}
-			}
-			else {
+			} else {
 				throw new EPPDecodeException(
 						"EPPUtil.decodeIntegerList Can't decode numeric value from non-existant text node");
 			}
@@ -1753,8 +1845,7 @@ public class EPPUtil {
 			if (Character.isWhitespace(c) == false) {
 				sb.append(c); // maintain all non-whitespace characters
 				prevWhitespace = false;
-			}
-			else {
+			} else {
 				if (prevWhitespace == false) {
 					sb.append(' '); // replace each sequence of whitespace with
 					// a single space
@@ -1765,8 +1856,7 @@ public class EPPUtil {
 
 		if (sb.length() == 0) {
 			return null; // return null string when result is empty
-		}
-		else {
+		} else {
 			return sb.toString(); // return collapsed string
 		}
 
@@ -1798,8 +1888,7 @@ public class EPPUtil {
 
 		if (sb.length() == 0) {
 			return null; // return null string when result is empty
-		}
-		else {
+		} else {
 			return sb.toString(); // return result string
 		}
 
@@ -1808,17 +1897,19 @@ public class EPPUtil {
 	/**
 	 * Base64 encode the XML <code>Element</code> tree.
 	 * 
-	 * @param aElement Root element to encode in Base64
+	 * @param aElement
+	 *            Root element to encode in Base64
 	 * @return Base64 encoded value of the XML <code>Element</code> tree.
-	 * @throws Exception Error encoding the <code>Element</code>
+	 * @throws Exception
+	 *             Error encoding the <code>Element</code>
 	 */
 	public static String encodeBase64(Element aElement) throws Exception {
 		ByteArrayOutputStream ostream = new ByteArrayOutputStream();
-		
+
 		TransformerFactory transFac = TransformerFactory.newInstance();
 		Transformer trans = transFac.newTransformer();
 		trans.transform(new DOMSource(aElement), new StreamResult(ostream));
-		
+
 		return (new String(Base64.encodeBase64(ostream.toByteArray(), true)));
 	}
 
