@@ -43,8 +43,6 @@ import org.apache.commons.codec.binary.Base64;
 import org.apache.log4j.Logger;
 import org.apache.xerces.dom.DocumentImpl;
 import org.w3c.dom.Document;
-import org.xml.sax.SAXNotRecognizedException;
-import org.xml.sax.SAXNotSupportedException;
 
 import com.verisign.epp.codec.domain.EPPDomainCheckCmd;
 import com.verisign.epp.codec.domain.EPPDomainCreateCmd;
@@ -126,7 +124,8 @@ public class LaunchDomainHandler extends DomainHandler {
 			System.exit(1);
 		}
 
-		cat.debug("LaunchDomainHandler.LaunchDomainHandler(): Signed Mark Truststore = " + truststore);
+		cat.debug("LaunchDomainHandler.LaunchDomainHandler(): Signed Mark Truststore = "
+				+ truststore);
 
 		// Load the CRL's
 		String crlsProp = Environment.getOption("EPP.SignedMark.crls");
@@ -214,7 +213,8 @@ public class LaunchDomainHandler extends DomainHandler {
 		FileInputStream trustStoreFile = new FileInputStream(aTrustStoreName);
 		trustStore.load(trustStoreFile, null);
 		trustStoreFile.close();
-		cat.debug("LaunchDomainHandler.loadPKIXParameters(String, String): truststore = " + aTrustStoreName);
+		cat.debug("LaunchDomainHandler.loadPKIXParameters(String, String): truststore = "
+				+ aTrustStoreName);
 		PKIXParameters pkixParameters = new PKIXParameters(trustStore);
 
 		CertificateFactory certFactory = CertificateFactory
@@ -228,7 +228,8 @@ public class LaunchDomainHandler extends DomainHandler {
 				InputStream inStream = null;
 
 				try {
-					cat.debug("LaunchDomainHandler.loadPKIXParameters(String, String): adding CRL " + currCrl);
+					cat.debug("LaunchDomainHandler.loadPKIXParameters(String, String): adding CRL "
+							+ currCrl);
 					inStream = new FileInputStream(currCrl);
 					crlContentsList.add(certFactory.generateCRL(inStream));
 				}
@@ -340,6 +341,7 @@ public class LaunchDomainHandler extends DomainHandler {
 							theLaunchCheck.getPhase());
 
 					boolean exists = true;
+					boolean validatorId = true;
 
 					Vector vDomainNames = theCommand.getNames();
 					Enumeration eDomainNames = vDomainNames.elements();
@@ -350,16 +352,35 @@ public class LaunchDomainHandler extends DomainHandler {
 
 						// Is there a matching mark for domain name?
 						if (exists) {
-							theExt.addCheckResult(new EPPLaunchCheckResult(
-									domainName, true, Base64
-											.encodeBase64String(domainName
-													.getBytes())));
-							exists = !exists;
+
+							// Include validatorID attribute?
+							if (validatorId) {
+								EPPLaunchCheckResult launchCheckResult = new EPPLaunchCheckResult(
+										domainName, true);
+								launchCheckResult.addClaimKey(Base64
+										.encodeBase64String(domainName
+												.getBytes()), "tmch");
+								launchCheckResult.addClaimKey(Base64
+										.encodeBase64String(domainName
+												.getBytes()), "custom-tmch");
+								theExt.addCheckResult(launchCheckResult);
+							}
+							else {
+								theExt.addCheckResult(new EPPLaunchCheckResult(
+										domainName, true, Base64
+												.encodeBase64String(domainName
+														.getBytes())));
+							}
+
+							validatorId = !validatorId;
+
 						} // No matching mark for domain name
 						else {
 							theExt.addCheckResult(new EPPLaunchCheckResult(
 									domainName, false));
 						}
+
+						exists = !exists;
 					}
 
 					theResponse.addExtension(theExt);
@@ -656,8 +677,6 @@ public class LaunchDomainHandler extends DomainHandler {
 
 				} // Signed marks passed?
 				else if (theCmdExt.hasSignedMarks()) {
-					
-					
 
 					List<EPPSignedMark> signedMarks = theCmdExt
 							.getSignedMarks();
@@ -670,7 +689,7 @@ public class LaunchDomainHandler extends DomainHandler {
 								theCommand.getTransId(),
 								"No signed marks provided");
 					}
-					
+
 					// If signed marks were passed via XML instead of Base64
 					// encoded?
 					if (!(signedMarks.get(0) instanceof EPPEncodedSignedMark)
@@ -684,12 +703,14 @@ public class LaunchDomainHandler extends DomainHandler {
 							theParser.setFeature(
 									EPPSchemaCachingParser.NORMALIZE_DATA,
 									false);
-							
+
 							SessionData sessionData = (SessionData) aData;
-														
-							byte[] packet = (byte []) sessionData.getAttribute("PACKET");
-							
-							cat.debug("Command with signed mark = [" + new String(packet) + "]");
+
+							byte[] packet = (byte[]) sessionData
+									.getAttribute("PACKET");
+
+							cat.debug("Command with signed mark = ["
+									+ new String(packet) + "]");
 
 							Document theDoc = theParser
 									.parse(new ByteArrayInputStream(packet));
@@ -713,10 +734,9 @@ public class LaunchDomainHandler extends DomainHandler {
 					// For each signed mark
 					for (EPPSignedMark signedMark : signedMarks) {
 
-						
 						cat.debug("LaunchDomainHandler.doDomainCreate: signedMark = ["
 								+ signedMark + "]");
-						
+
 						Document doc = new DocumentImpl();
 						String signedMarkXML;
 						try {
@@ -736,7 +756,7 @@ public class LaunchDomainHandler extends DomainHandler {
 						System.out
 								.println("\nLaunchDomainHandler.doDomainCreate: base64 signed mark = \n"
 										+ new String(Base64.encodeBase64(
-												signedMarkXML.getBytes(), true)));						
+												signedMarkXML.getBytes(), true)));
 
 						// Signature valid?
 						if (signedMark.validate(this.pkixParameters)) {
@@ -749,7 +769,7 @@ public class LaunchDomainHandler extends DomainHandler {
 									theCommand.getTransId(),
 									"Signed mark signature invalid");
 						}
-						
+
 						// SMD is not revoked?
 						if (!this.smdRevocationList.isRevoked(signedMark)) {
 							cat.debug("LaunchDomainHandler.doDomainCreate: signed mark is not revoked");
@@ -761,7 +781,6 @@ public class LaunchDomainHandler extends DomainHandler {
 									theCommand.getTransId(),
 									"Signed mark is revoked");
 						}
-						
 
 					}
 
@@ -944,16 +963,21 @@ public class LaunchDomainHandler extends DomainHandler {
 				// Notice information included?
 				if (theCmdExt.hasNotice()) {
 
-					EPPLaunchNotice notice = theCmdExt.getNotice();
+					for (EPPLaunchNotice notice : theCmdExt.getNotices()) {
+						if (notice.hasValidatorId()) {
+							cat.debug("LaunchDomainHandler.doDomainCreate: validator id = ["
+									+ notice.getValidatorId() + "]");
+						}
+						cat.debug("LaunchDomainHandler.doDomainCreate: notice id = ["
+								+ notice.getNoticeId() + "]");
+						cat.debug("LaunchDomainHandler.doDomainCreate: notice not after date = ["
+								+ EPPUtil.encodeTimeInstant(notice
+										.getNotAfterDate()) + "]");
+						cat.debug("LaunchDomainHandler.doDomainCreate: notice accepted date = ["
+								+ EPPUtil.encodeTimeInstant(notice
+										.getAcceptedDate()) + "]");
+					}
 
-					cat.debug("LaunchDomainHandler.doDomainCreate: notice id = ["
-							+ notice.getNoticeId() + "]");
-					cat.debug("LaunchDomainHandler.doDomainCreate: notice not after date = ["
-							+ EPPUtil.encodeTimeInstant(notice
-									.getNotAfterDate()) + "]");
-					cat.debug("LaunchDomainHandler.doDomainCreate: notice accepted date = ["
-							+ EPPUtil.encodeTimeInstant(notice
-									.getAcceptedDate()) + "]");
 				}
 
 				// Is application?
